#pragma once

#include "cv.h"
#include "cxmisc.h"

class CVRECT2 : public CvRect
{
public:
	CVRECT2()
	{
		x=y=width=height=x_Right=y_Bottom=area=0;
		x_Center=y_Center=0;
	}
	CVRECT2(const CvRect & rect)
	{
		convert_from_CvRect(rect);
	}
	int overlapped_area(CVRECT2 & rect2)
	{
		int delta_x,delta_y;
		const int max_x1=CV_IMAX(x,rect2.x);
		const int max_y1=CV_IMAX(y,rect2.y);
		const int min_x2=CV_IMIN(x_Right,rect2.x_Right);
		const int min_y2=CV_IMIN(y_Bottom,rect2.y_Bottom);
		delta_x=min_x2-max_x1+1;
		delta_y=min_y2-max_y1+1;
		if(delta_x>=1 && delta_y>=1)
			return delta_x*delta_y;
		else
			return 0;
	}

	void convert_from_CvRect(const CvRect & rect)
	{
		x = rect.x;
		y = rect.y;
		x_Right = rect.x + rect.width - 1;
		y_Bottom = rect.y + rect.height - 1;
		width = rect.width;
		height = rect.height;
		area = rect.width*rect.height;
		x_Center=rect.x + double(rect.width-1)/2;
		y_Center=rect.y + double(rect.height-1)/2;
	}

public:
	int area;
	int x_Right,y_Bottom;
	float x_Center,y_Center;
};




class Histogram
{
public:
	Histogram()
	{
		hist_size=hist_size_max=0;
		hist_buf=0;hist_data_ptr=0;
	}
	~Histogram()
	{
		if(hist_buf) cvFree(&hist_buf);
	}
	void set(float e)
	{
		assert(hist_size>0);
		const int L = hist_size;
		float * d = hist_data_ptr;
		for (int i=0; i<L; i++)
			d[i]=e;
	}
	int copy_from(Histogram * pHist)
	{
		if (hist_size <=0 && pHist->hist_size!=hist_size && hist_data_ptr==0 && pHist->hist_data_ptr==0)
			return 0;
		const int L = hist_size;
		float * d = hist_data_ptr;
		float * s = pHist->hist_data_ptr;
		for (int i=0; i<L; i++)
			d[i] = s[i];
		return 1;
	}
	float sum()
	{
		assert(hist_size>0);
		float sum = 0;
		const int L = hist_size;
		float * d = hist_data_ptr;
		for (int i=0; i<L; i++)
		{
			sum += d[i];
		}
		return sum;
	}
	void normalize()
	{
		assert(hist_size>0);
		float sum = this->sum();
		if(sum!=0.f && sum!=1.f)
		{
			sum = 1.0/sum;
			const int L = hist_size;
			float * d = hist_data_ptr;
			for(int i=0; i<L; i++ )
			{
				d[i] = d[i] * sum;
			}
		}
	}

	int calc_2D(CvMat *pMat, CvRect rect)
	{
		if(!CV_IS_MAT(pMat))
			return 0;
		int src_type;
		src_type = CV_MAT_TYPE( pMat->type );
		
		if(src_type==CV_8UC1 || src_type==CV_8UC2)
		{
			int x,y,height_Max,width_Max;
			set(0);
			if(rect.x<=0)
				rect.x=0;
			if(rect.y<=0)
				rect.y=0;
			height_Max = pMat->rows;
			if (rect.y+rect.height<height_Max)
				height_Max = rect.y+rect.height;
			width_Max = pMat->cols;
			if (rect.x+rect.width<width_Max)
				width_Max = rect.x+rect.width;

			float * d = hist_data_ptr;
			const int L = hist_size;
			const int L_M = hist_size_max;
			const int T = width_Max-rect.x;
			const int width = (src_type==CV_8UC1)?T:2*T;
			//const int height = height_Max-rect.y;
			uchar * sdata = (uchar *)CV_MAT_ELEM_PTR(*pMat,rect.y,rect.x);
			for(y=rect.y;y<height_Max;y++,sdata+=pMat->step)
			{
				uchar * s = sdata;
				for(x=0;x<width;x++)
				{
					assert( L>0 && s[x]<L_M);
					d[s[x]] += 1;
				}
			}
			return 1;
		}
		else
		{
			return 0;
		}
	}

	double distance_from_2n(Histogram * pHist)
	{
		assert (hist_size>0 && pHist->hist_size==hist_size && hist_data_ptr!=0 && pHist->hist_data_ptr!=0);
		float * d1 = hist_data_ptr;
		float * d2 = pHist->hist_data_ptr;
		double result = 0;
		for(int i=0; i<hist_size; i++ )
		{
			double a = d1[i];
			double b = d2[i];
			result += sqrt(a*b);
		}
		result = 1. - result;
		result = exp(-20*MAX(result,0.));
		return result;
	}

	int init(int length)
	{
		if (length <=0)
		{
			return 0;
		}
		if(length!=hist_size)
		{
			if(hist_buf) cvFree(&hist_buf);
			hist_size_max = hist_size = length;
			int hist_buf_size = cvAlign((length*sizeof(float)),CV_MALLOC_ALIGN);;
			hist_buf = (uchar *)cvAlloc(hist_buf_size);
			if(hist_buf==0)
			{
				return 0;
			}
			hist_data_ptr = (float*)hist_buf;
		}
		set(0);

		return 1;
	}

private:
	uchar * hist_buf;
	int hist_size;
	int hist_size_max;
	float * hist_data_ptr;
};




/****************************************************************************************\
  
  Generic implementation of QuickSort algorithm.
  ----------------------------------------------
  Using this macro user can declare customized sort function that can be much faster
  than built-in qsort function because of lower overhead on elements
  comparison and exchange. The macro takes less_than (or LT) argument - a macro or function
  that takes 2 arguments returns non-zero if the first argument should be before the second
  one in the sorted sequence and zero otherwise.

  Example:

    Suppose that the task is to sort points by ascending of y coordinates and if
    y's are equal x's should ascend.

    The code is:
    ------------------------------------------------------------------------------
           #define cmp_pts( pt1, pt2 ) \
               ((pt1).y < (pt2).y || ((pt1).y < (pt2).y && (pt1).x < (pt2).x))

           [static] CV_IMPLEMENT_QSORT( icvSortPoints, CvPoint, cmp_pts )
    ------------------------------------------------------------------------------

    After that the function "void icvSortPoints( CvPoint* array, size_t total, int aux );"
    is available to user.

  aux is an additional parameter, which can be used when comparing elements.
  The current implementation was derived from *BSD system qsort():

    * Copyright (c) 1992, 1993
    *  The Regents of the University of California.  All rights reserved.
    *
    * Redistribution and use in source and binary forms, with or without
    * modification, are permitted provided that the following conditions
    * are met:
    * 1. Redistributions of source code must retain the above copyright
    *    notice, this list of conditions and the following disclaimer.
    * 2. Redistributions in binary form must reproduce the above copyright
    *    notice, this list of conditions and the following disclaimer in the
    *    documentation and/or other materials provided with the distribution.
    * 3. All advertising materials mentioning features or use of this software
    *    must display the following acknowledgement:
    *  This product includes software developed by the University of
    *  California, Berkeley and its contributors.
    * 4. Neither the name of the University nor the names of its contributors
    *    may be used to endorse or promote products derived from this software
    *    without specific prior written permission.
    *
    * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
    * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
    * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
    * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
    * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
    * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    * SUCH DAMAGE.

\****************************************************************************************/

#define CV_IMPLEMENT_QSORT_EX( func_name, T, LT, user_data_type )                   \
void func_name( T *array, size_t total, user_data_type aux )                        \
{                                                                                   \
    int isort_thresh = 7;                                                           \
    T t;                                                                            \
    int sp = 0;                                                                     \
                                                                                    \
    struct                                                                          \
    {                                                                               \
        T *lb;                                                                      \
        T *ub;                                                                      \
    }                                                                               \
    stack[48];                                                                      \
                                                                                    \
    aux = aux;                                                                      \
                                                                                    \
    if( total <= 1 )                                                                \
        return;                                                                     \
                                                                                    \
    stack[0].lb = array;                                                            \
    stack[0].ub = array + (total - 1);                                              \
                                                                                    \
    while( sp >= 0 )                                                                \
    {                                                                               \
        T* left = stack[sp].lb;                                                     \
        T* right = stack[sp--].ub;                                                  \
                                                                                    \
        for(;;)                                                                     \
        {                                                                           \
            int i, n = (int)(right - left) + 1, m;                                  \
            T* ptr;                                                                 \
            T* ptr2;                                                                \
                                                                                    \
            if( n <= isort_thresh )                                                 \
            {                                                                       \
            insert_sort:                                                            \
                for( ptr = left + 1; ptr <= right; ptr++ )                          \
                {                                                                   \
                    for( ptr2 = ptr; ptr2 > left && LT(ptr2[0],ptr2[-1]); ptr2--)   \
                        CV_SWAP( ptr2[0], ptr2[-1], t );                            \
                }                                                                   \
                break;                                                              \
            }                                                                       \
            else                                                                    \
            {                                                                       \
                T* left0;                                                           \
                T* left1;                                                           \
                T* right0;                                                          \
                T* right1;                                                          \
                T* pivot;                                                           \
                T* a;                                                               \
                T* b;                                                               \
                T* c;                                                               \
                int swap_cnt = 0;                                                   \
                                                                                    \
                left0 = left;                                                       \
                right0 = right;                                                     \
                pivot = left + (n/2);                                               \
                                                                                    \
                if( n > 40 )                                                        \
                {                                                                   \
                    int d = n / 8;                                                  \
                    a = left, b = left + d, c = left + 2*d;                         \
                    left = LT(*a, *b) ? (LT(*b, *c) ? b : (LT(*a, *c) ? c : a))     \
                                      : (LT(*c, *b) ? b : (LT(*a, *c) ? a : c));    \
                                                                                    \
                    a = pivot - d, b = pivot, c = pivot + d;                        \
                    pivot = LT(*a, *b) ? (LT(*b, *c) ? b : (LT(*a, *c) ? c : a))    \
                                      : (LT(*c, *b) ? b : (LT(*a, *c) ? a : c));    \
                                                                                    \
                    a = right - 2*d, b = right - d, c = right;                      \
                    right = LT(*a, *b) ? (LT(*b, *c) ? b : (LT(*a, *c) ? c : a))    \
                                      : (LT(*c, *b) ? b : (LT(*a, *c) ? a : c));    \
                }                                                                   \
                                                                                    \
                a = left, b = pivot, c = right;                                     \
                pivot = LT(*a, *b) ? (LT(*b, *c) ? b : (LT(*a, *c) ? c : a))        \
                                   : (LT(*c, *b) ? b : (LT(*a, *c) ? a : c));       \
                if( pivot != left0 )                                                \
                {                                                                   \
                    CV_SWAP( *pivot, *left0, t );                                   \
                    pivot = left0;                                                  \
                }                                                                   \
                left = left1 = left0 + 1;                                           \
                right = right1 = right0;                                            \
                                                                                    \
                for(;;)                                                             \
                {                                                                   \
                    while( left <= right && !LT(*pivot, *left) )                    \
                    {                                                               \
                        if( !LT(*left, *pivot) )                                    \
                        {                                                           \
                            if( left > left1 )                                      \
                                CV_SWAP( *left1, *left, t );                        \
                            swap_cnt = 1;                                           \
                            left1++;                                                \
                        }                                                           \
                        left++;                                                     \
                    }                                                               \
                                                                                    \
                    while( left <= right && !LT(*right, *pivot) )                   \
                    {                                                               \
                        if( !LT(*pivot, *right) )                                   \
                        {                                                           \
                            if( right < right1 )                                    \
                                CV_SWAP( *right1, *right, t );                      \
                            swap_cnt = 1;                                           \
                            right1--;                                               \
                        }                                                           \
                        right--;                                                    \
                    }                                                               \
                                                                                    \
                    if( left > right )                                              \
                        break;                                                      \
                    CV_SWAP( *left, *right, t );                                    \
                    swap_cnt = 1;                                                   \
                    left++;                                                         \
                    right--;                                                        \
                }                                                                   \
                                                                                    \
                if( swap_cnt == 0 )                                                 \
                {                                                                   \
                    left = left0, right = right0;                                   \
                    goto insert_sort;                                               \
                }                                                                   \
                                                                                    \
                n = MIN( (int)(left1 - left0), (int)(left - left1) );               \
                for( i = 0; i < n; i++ )                                            \
                    CV_SWAP( left0[i], left[i-n], t );                              \
                                                                                    \
                n = MIN( (int)(right0 - right1), (int)(right1 - right) );           \
                for( i = 0; i < n; i++ )                                            \
                    CV_SWAP( left[i], right0[i-n+1], t );                           \
                n = (int)(left - left1);                                            \
                m = (int)(right1 - right);                                          \
                if( n > 1 )                                                         \
                {                                                                   \
                    if( m > 1 )                                                     \
                    {                                                               \
                        if( n > m )                                                 \
                        {                                                           \
                            stack[++sp].lb = left0;                                 \
                            stack[sp].ub = left0 + n - 1;                           \
                            left = right0 - m + 1, right = right0;                  \
                        }                                                           \
                        else                                                        \
                        {                                                           \
                            stack[++sp].lb = right0 - m + 1;                        \
                            stack[sp].ub = right0;                                  \
                            left = left0, right = left0 + n - 1;                    \
                        }                                                           \
                    }                                                               \
                    else                                                            \
                        left = left0, right = left0 + n - 1;                        \
                }                                                                   \
                else if( m > 1 )                                                    \
                    left = right0 - m + 1, right = right0;                          \
                else                                                                \
                    break;                                                          \
            }                                                                       \
        }                                                                           \
    }                                                                               \
}

#define CV_IMPLEMENT_QSORT( func_name, T, cmp )  \
    CV_IMPLEMENT_QSORT_EX( func_name, T, cmp, int )
