#pragma once

#include "misc.h"

// The particle (sample) of SIMM filter
class CvSample
{
public:
	CvSample(void)
	{
		pc_weight=1;
		pc_likelihood=0;
		x=y=s_w=s_h=0;
		v_x=v_y=FLT_EPSILON;
		pc_Repel_weight=0;
	}

public:
	unsigned int get_random_dimensions(){return 2;}
	int * get_work_buffer_bytes(){return 0;}

public:
	void MO_move_random_walk(float r_x, float r_y, float r_sw=0.0, float r_sh=0.0)
	{
		float new_x=x+v_x+r_x;
		float new_y=y+v_y+r_y;
		v_x=(new_x-x);
		v_y=(new_y-y);
		x=new_x;
		y=new_y;
		s_w=s_w+r_sw;
		s_h=s_h+r_sh;
	}
	CvRect MO_get_rect(int SC_MIN_X,int SC_MIN_Y,int SC_MAX_X,int SC_MAX_Y,
		float w, float h)
	{
		CvRect rect;
		const float w1=w*s_w;
		const float h1=h*s_h;
		rect.x = cvRound(x-0.5*w1);
		rect.y = cvRound(y-0.5*h1);
		rect.width = cvRound(w1);
		rect.height = cvRound(h1);
		rect.x = CV_IMIN(SC_MAX_X-1,CV_IMAX(SC_MIN_X,rect.x));
		rect.y = CV_IMIN(SC_MAX_Y-1,CV_IMAX(SC_MIN_Y,rect.y));
		rect.width = CV_IMAX(1,CV_IMIN(SC_MAX_X-rect.x,rect.width));
		rect.height = CV_IMAX(1,CV_IMIN(SC_MAX_Y-rect.y,rect.height));
		assert(rect.width>0 && rect.height>0);
		return rect;
	}
	void MO_zero(void)
	{
		x=y=0;
		s_w=s_h=0;
		v_x=v_y=FLT_EPSILON;}
	void MO_acc(CvSample & other)
	{
		x+=other.x;y+=other.y;
		v_x+=other.v_x;v_y+=other.v_y;
		s_w+=other.s_w;s_h+=other.s_h;
	}
	void MO_mul(float c)
	{
		x*=c;y*=c;
		v_x*=c;v_y*=c;
		s_w*=c;s_h*=c;
	}
	void MO_add_mul(CvSample other,float c)
	{
		x+=(other.x*c);y+=(other.y*c);
		v_x+=(other.v_x*c);v_y+=(other.v_y*c);
		s_w+=(other.s_w*c);s_h+=(other.s_h*c);
	}


public:
	// The weight of particle
	float pc_weight;
	// The likelihood of particle
	float pc_likelihood;
	// The rectangle of particle
	CVRECT2 pc_rect2;
	CvRect rect;
	// The state of target
	float x,y,v_x,v_y,s_w,s_h;
	//  CvPseudoJointParticle
	float pc_Repel_weight;
};


