#pragma once

#include <vector>
#include <algorithm>
#include <functional>

#include "cxmisc.h"

#include "sample.h"
#include "random.h"
#include "target.h"

#define cmp_particle_by_weight( p1, p2 ) ( (p1)->pc_weight > (p2)->pc_weight )
const int max_object_number = 10;

const int m_pj_iteration_size = 5000;
const double c_alpha = 8;
const double m0_weight = 0.5;
const double m1_weight = 0.5;

// semi-independent joint motion model (SIMM) particle filter
// Author: Lejun Shen
class simm
{
public:
	simm(void)
	{
		m_max_sample_size=0; m_sample_set_buffer_size=0;
		m_random_ptrs = 0; m_sample_ptr = 0; 
		m_random_ptrs2 = 0; m_sample_ptr2 = 0;
		m_buffer =0; m_last_start=-1;
		m_target_of_current_subset=0;
		m_particle_ptrs=m_particle_ptrs2=0;
		m_particle_filter_buffer=0;m_temp_particle_ptr=m_temp_particle_ptr2=0;
		m_lut_size=0;m_particle_size=m_max_number_of_object=0;
		hue_max = 179;
		cvInitFont( &mo_font, CV_FONT_HERSHEY_PLAIN,1.0,1.0,0,1,8);
		cvGetTextSize("A",&mo_font,&mo_font_size,0);
	}
	~simm(void){;}

	void resample(int length)
	{
		const int particle_bytes = get_particle_bytes();
		assert((m_last_start+length)*particle_bytes<=m_sample_set_buffer_size);
		assert(length>0);
		CvSample ** particle_ptrs = get_particle_ptrs();
		CvSample * particle_ptr = get_particle_ptr();
		CvSample * temp_particle_ptr = get_temp_particle_ptr();

		int i,k;
		double sum = 0;
		for(i=0;i<length; i++)
		{
			sum+=particle_ptrs[i]->pc_weight;
		}
		for (i=0,k=0; i<length; i++)
		{
			const int np = cvRound( particle_ptrs[i]->pc_weight * length / sum);
			if(np>0)
			{
				for(int j = 0; j < np && k<length; j++, k++) 
					copy_particles(particle_ptrs[i],&temp_particle_ptr[k]);
			}
		}
		for(i=0; k < length; i++,k++)
		{
			copy_particles(particle_ptrs[i],&temp_particle_ptr[k]);
		}

		copy_particles(&temp_particle_ptr[0],&particle_ptr[0],length);
	}

	int on_generate_random_all(int length)
	{
		CvRandomGenerator *rg = &m_rg;
		unsigned char ** rs = get_random_ptrs();

		const float v0 = m_system_variance[0];
		const float v1 = m_system_variance[1];
		rg->randn_float((float *)rs[0],0,length);
		rg->randn_float((float *)rs[1],1,length);
		const int L = length;
		float * r0 = (float *)rs[0];
		float * r1 = (float *)rs[1];
		for(int i=0;i<L;i++) r0[i]*=v0;
		for(int i=0;i<L;i++) r1[i]*=v1;

		return 1;
	}
	// Eq. (11)
	void on_transition(int length)
	{
		unsigned char ** rs = get_random_ptrs();
		CvSample * particle_ptr = get_sample_ptr();
		Target * p_vp=(Target *)get_variable_parameter_of_current_subset();

		const int L = length;
		const float * r0 = (float *)rs[0];
		const float * r1 = (float *)rs[1];
		for(int i=0;i<L;i++)
		{
			particle_ptr[i].MO_move_random_walk(r0[i],r1[i]);
			const CvRect rect = particle_ptr[i].MO_get_rect(0,0,m_screen_width,m_screen_height,
				p_vp->get_width(),p_vp->get_height());
			particle_ptr[i].rect = rect;
			particle_ptr[i].pc_rect2.convert_from_CvRect(rect);
		}
	}
	// Eq.(12)
	void on_observation(int length)
	{
		CvSample * particle_ptr = get_sample_ptr();
		Target * target=(Target *)get_variable_parameter_of_current_subset();
		const int L = length;
		for(int i=0;i<L;i++)
		{
			const float likelihood = target->likelihood_computing(particle_ptr[i].rect,image);
			particle_ptr[i].pc_weight = particle_ptr[i].pc_likelihood = likelihood + FLT_EPSILON;
		}
	}

	void detect_interaction(void)
	{
		if(get_targets()->size()>0)
		{
			std::vector<Target *>::iterator it;
			for(it = get_targets()->begin();
				it != get_targets()->end(); 
				it++)
			{
				const int length = (*it)->get_length();
				build_subset_ptrs((*it)->get_subset_start());
				CvSample * particle_ptr = get_sample_ptr();

				int j;
				(*it)->pj_outer_rect2=particle_ptr[0].pc_rect2;
				for(j=1; j<length; j++) 
				{
					(*it)->pj_outer_rect2.x = CV_IMIN(particle_ptr[j].pc_rect2.x,(*it)->pj_outer_rect2.x);
					(*it)->pj_outer_rect2.y = CV_IMIN(particle_ptr[j].pc_rect2.y,(*it)->pj_outer_rect2.y);
					(*it)->pj_outer_rect2.x_Right = CV_IMAX(particle_ptr[j].pc_rect2.x_Right,(*it)->pj_outer_rect2.x_Right);
					(*it)->pj_outer_rect2.y_Bottom = CV_IMAX(particle_ptr[j].pc_rect2.y_Bottom,(*it)->pj_outer_rect2.y_Bottom);
				}
				for(j=0; j<length; j++)
				{
					particle_ptr[j].pc_Repel_weight=FLT_EPSILON;
				}
				(*it)->pj_interaction_flag=0;
			}

			it = get_targets()->begin();
			for(; it != get_targets()->end(); it++)
			{
				std::vector<Target *>::iterator it2;
				(*it)->pj_overlaped_objects.clear();

				it2 = get_targets()->begin();
				while(it2!=get_targets()->end())
				{
					if((*it)->get_id()!=(*it2)->get_id())
						if((*it2)->pj_outer_rect2.overlapped_area((*it)->pj_outer_rect2))
						{
							(*it)->pj_interaction_flag=(*it2)->pj_interaction_flag=1;
							(*it)->pj_overlaped_objects.push_back((*it2));
						}
					it2++;
				}
			}
		}
	}

	void pj_sampling(int pj_sampling_size=-1)
	{
		if(pj_sampling_size==-1)
			pj_sampling_size=m_pj_iteration_size;
		if(pj_sampling_size>0 && get_targets()->size()>0)
		{
			std::vector<Target *>::iterator it;
			it = get_targets()->begin();
			int i;
			for(i=0; i<get_targets()->size(); i++,it++)
			{
				const int length = (*it)->get_length();
				build_subset_ptrs((*it)->get_subset_start());
				CvSample * particle_ptr = get_sample_ptr();
				CvSample ** bootstrap_ptrs = (CvSample **)(*it)->get_bootstrap_bufs();
				assert(bootstrap_ptrs);
				int j,l; 
				double sum=0.f;
				for(j=0; j<length; j++) 
				{
					sum+=particle_ptr[j].pc_weight;
				}
				double z=sum/m_lut_size,s=0.f;
				for(l=0,j=0;j<length && l<=m_lut_size;j++)
				{
					s+=particle_ptr[j].pc_weight;
					while(l*z<s)
						bootstrap_ptrs[l++]=&particle_ptr[j];
				}
				while(l<=m_lut_size)
					bootstrap_ptrs[l++]=&particle_ptr[0];
			}
			_pj_sampling(pj_sampling_size);
		}
	}

	virtual void _pj_sampling(int pj_sampling_size)
	{
		std::vector<Target *>::iterator it;
		int i;
		for(i=0;i<pj_sampling_size;i++)
		{
			int index;
			// Eq. (9) or (14)
			it = get_targets()->begin();
			for(index=0; index<get_targets()->size(); index++, it++)
			{
				CvRNG * rngs = m_rg2.get_rng();
				CvSample ** bootstrap_ptrs = (CvSample **)(*it)->get_bootstrap_bufs();
				uint64 temp = rngs[index];
				temp = ICV_RNG_NEXT(temp);
				const int r = (unsigned)temp & m_lut_size;
				rngs[index]=temp;
				(*it)->pj_particle = bootstrap_ptrs[r];
			}
			// MRF
			it = get_targets()->begin();
			for(index=0; index<get_targets()->size(); index++, it++)
			{
				if((*it)->pj_overlaped_objects.size())
				{
					int interacted = 0;
					double MRF_value=1.0;
					// Eq. (15)
					std::vector< Target * >::iterator it2;
					it2 = (*it)->pj_overlaped_objects.begin();
					while(it2!=(*it)->pj_overlaped_objects.end())
					{
						double f;
						int o = (*it)->pj_particle->pc_rect2.overlapped_area((*it2)->pj_particle->pc_rect2);
						if(o==0) 
							f=1.0;
						else
						{
							assert(o<=(*it)->pj_particle->pc_rect2.area);
							const double a = double(o)/(*it)->pj_particle->pc_rect2.area;
							f=exp(-a*a*c_alpha*c_alpha);
							interacted=1;
						}
						MRF_value=MRF_value*f;
						it2++;
					}
					// Eq. (18)
					if(interacted)
						(*it)->pj_particle->pc_Repel_weight+=MRF_value;
					else
						(*it)->pj_particle->pc_Repel_weight+=1.0;
				}
				else
					(*it)->pj_particle->pc_Repel_weight+=1.0;
			}
		}

		it = get_targets()->begin();
		for(i=0; i<get_targets()->size(); i++,it++)
		{
			double sum_of_pc_repel_weight=0.f;
			double sum_of_pc_likelihood=0.f;
			const int length = (*it)->get_length();
			build_subset_ptrs((*it)->get_subset_start());
			CvSample * particle_ptr = get_particle_ptr();
			int j;
			for( j=0 ; j<length; j++)
			{
				sum_of_pc_repel_weight += particle_ptr[j].pc_Repel_weight;
				sum_of_pc_likelihood += particle_ptr[j].pc_likelihood;
			}
			// Eq. (19)
			for( j=0 ; j<length; j++)
			{
				double m0 = m0_weight*(particle_ptr[j].pc_likelihood/sum_of_pc_likelihood);
				double m1 = m1_weight*(particle_ptr[j].pc_Repel_weight/sum_of_pc_repel_weight);
				particle_ptr[j].pc_weight = m0 + m1;
			}
		}
	}

	// particle filter
	int process()
	{
		if (get_targets()->size()) detect_interaction();
		assert(m_max_sample_size!=0);

		std::vector<Target *>::iterator it;
		const int M = get_targets()->size();
		it = get_targets()->begin();
		for(int i=0; i<M; i++,it++)
		{
			const int length = (*it)->get_length();
			build_subset_ptrs((*it)->get_subset_start(),(*it));
			resample(length);
			reset_particle_ptrs(length);
			on_generate_random_all(length);
			on_transition(length);
			on_observation(length);
			sort_particle_ptrs_by_weight(length);
		}

		if (get_targets()->size()) pj_sampling();
		return 1;
	}

	int init0(int max_sample_size)
	{
		int N_R,i;
		if(max_sample_size>0 && m_max_sample_size==0)
			m_max_sample_size = max_sample_size;
		else
			return 0;

		CvSample sample;
		N_R = sample.get_random_dimensions();
		m_rg.init(N_R);
		const int sample_objs_buf_size = cvAlign(m_max_sample_size*sizeof(CvSample),CV_MALLOC_ALIGN);
		int random_buf_size = 0;
		for(i=0;i<N_R;i++)
		{
			int s = cvAlign(m_max_sample_size*sizeof(float),CV_MALLOC_ALIGN);
			random_buf_size+=s;
		}
		const int random_ptrs_buf_size = cvAlign(N_R*sizeof(unsigned char *),CV_MALLOC_ALIGN);
		int total_buf_size = sample_objs_buf_size + random_ptrs_buf_size*2 + random_buf_size;
		m_buffer = (unsigned char *)cvAlloc(total_buf_size);
		if(m_buffer)
		{
			for(i=0;i<total_buf_size;i++) m_buffer[i]=0;
			m_sample_ptr = (CvSample *)m_buffer;
			for(i=0;i<m_max_sample_size;i++)
				m_sample_ptr[i]=sample;
			m_sample_ptr2 = m_sample_ptr;
			m_random_ptrs = (unsigned char **)(m_buffer+sample_objs_buf_size);
			m_random_ptrs2 = (unsigned char **)(m_buffer+sample_objs_buf_size+random_ptrs_buf_size);
			unsigned char * random_ptr = (unsigned char *)(m_buffer+sample_objs_buf_size+random_ptrs_buf_size*2);
			m_random_ptrs[0]=m_random_ptrs2[0]=random_ptr;
			for(i=1;i<N_R;i++)
			{
				int s = cvAlign(m_max_sample_size*sizeof(float),CV_MALLOC_ALIGN);
				m_random_ptrs[i]=m_random_ptrs2[i]=random_ptr+s;
				random_ptr+=s;
			}
			m_sample_set_buffer_size = total_buf_size;

			const int temp_particle_ptr_buf_size = cvAlign(m_max_sample_size*sizeof(CvSample),CV_MALLOC_ALIGN);
			const int particle_ptrs_buf_size = cvAlign(m_max_sample_size*sizeof(CvSample *)*2,CV_MALLOC_ALIGN);
			total_buf_size = temp_particle_ptr_buf_size+particle_ptrs_buf_size;
			m_particle_filter_buffer = (unsigned char *)cvAlloc(total_buf_size);
			if(m_particle_filter_buffer)
			{
				for(int i=0;i<total_buf_size;i++) m_particle_filter_buffer[i]=0;
				m_temp_particle_ptr = (CvSample *)(m_particle_filter_buffer);
				m_particle_ptrs = (CvSample **)(m_particle_filter_buffer+temp_particle_ptr_buf_size);
				m_particle_ptrs2 = m_particle_ptrs;
				reset_particle_ptrs(m_max_sample_size);
				return 1;
			}
			return 1;
		}
		else
			return 0;
	}
	int pj_init(int particle_size, int max_number_of_object, int lut_size = ((1<<9)-1))
	{
		assert(particle_size>1 && max_number_of_object>0 && lut_size>1);

		const int max_sample_size = particle_size*max_number_of_object;
		if(!((lut_size+1) & lut_size ) == 0)
			return 0;
		if(init0(max_sample_size))
		{
			m_lut_size = lut_size;
			m_particle_size = particle_size;
			m_max_number_of_object = max_number_of_object;
			m_rg2.init(max_number_of_object);
			set_subset_default_particle_size(particle_size);
			return 1;
		}
		return 0;
	}
	Target * subset_new(int length=-1,CvSample * new_particle=0)
	{
		assert(m_max_sample_size!=0);
		if(length==-1) length=m_subset_default_particle_size;
		int start=0;
		if(m_Targets.size()!=0)
		{
			std::vector<Target *>::iterator it;
			it = get_targets()->end();
			it --;
			start=(*it)->get_subset_next_start();
			if((start+length)>m_max_sample_size)
				return 0;
		}
		else
			if(length>m_max_sample_size)
				return 0;
		Target * new_vp = new Target;
		assert(new_vp);
		new_vp->set_subset_idx(start,length);
		new_vp->assign_new_id();
		m_Targets.push_back(new_vp);
		return new_vp; 
	}
	Target * pj_new(CvSample * new_particle=0)
	{
		Target * vp = subset_new(-1,new_particle);
		vp->init_pj_buf(m_lut_size,m_max_number_of_object);
		return vp;
	}

	int init_tracker(CvMat *image,int hue_max,int particle_size)
	{
		if (image==0) return 0;
		int lut_size= ((1<<18)-1); //18
		pj_init(particle_size,max_object_number,lut_size);
		set_subset_default_particle_size(particle_size);
		m_screen_width = image->cols;
		m_screen_height = image->rows;
		return 1;
	}

	int new_target(CvMat * src_image, int x,int y,int w,int h,int gt_object_id, int id=-1)
	{ 
		if (src_image==0 || h==0 || w==0)
			return 0;

		CvSample particle;
		particle.x = x + w*0.5;
		particle.y = y + h*0.5;
		particle.s_w = particle.s_h = 1.0;
		particle.v_x = particle.v_y = 0;
		particle.pc_likelihood=particle.pc_weight = 1.f;
		CvRect rect;
		rect.x = x;rect.y=y;rect.width=w;rect.height=h;
		particle.rect = rect;
		particle.pc_rect2.convert_from_CvRect(rect);

		Target * p_vp = pj_new(&particle);
		if(p_vp==0) return 0;
		subset_set(p_vp->get_id(),particle);
		p_vp->set_height(h);
		p_vp->set_width(w);

		const int subset_length = p_vp->get_length();
		build_subset_ptrs(p_vp->get_subset_start(),p_vp);
		on_generate_random_all(subset_length);
		on_transition(subset_length);
		p_vp->likelihood_init(hue_max);
		p_vp->likelihood_copy_to_reference(rect,src_image);
		p_vp->m_mean = particle;
		on_observation(subset_length);
		return 1;
	}
	virtual void show(CvMat *dst_image)
	{
		assert(dst_image);
		int j;
		std::vector<Target *>::iterator iter;
		std::vector<Target *> * vp = get_targets();
		if(vp->size()==0)
			return;

		iter = vp->begin();
		for(j=0; j<vp->size(); j++,iter++)
		{
			build_subset_ptrs((*iter)->get_subset_start());

			int L = (*iter)->get_subset_length();
			int i;
			CvSample ** particle_ptrs = get_particle_ptrs();
			for (i = 0; i < L ; i++)
			{
				float w=(*iter)->get_width()*particle_ptrs[i]->s_w;
				float h=(*iter)->get_height()*particle_ptrs[i]->s_h;
				CvPoint pt_top_left,pt_bottom_right;
				pt_top_left.x = cvRound(particle_ptrs[i]->x-0.5*w);
				pt_top_left.y = cvRound(particle_ptrs[i]->y-0.5*h);
				pt_bottom_right.x = pt_top_left.x + w - 1;
				pt_bottom_right.y = pt_top_left.y + h - 1;
				mo_draw_rectangle(dst_image,pt_top_left, pt_bottom_right,CV_RGB(255,0,0));
			}

			CvSample m_mean;
			double m_mean_weight;
			m_mean.MO_zero();m_mean_weight=0;
			for (i = 0; i < L; i++)
			{
				CvSample t;
				t=*(particle_ptrs[i]);
				t.MO_mul(particle_ptrs[i]->pc_weight);
				m_mean.MO_acc(t);
				m_mean_weight+=particle_ptrs[i]->pc_weight;
			}
			double inv_w = 1/m_mean_weight;
			m_mean.MO_mul(inv_w);
			(*iter)->m_mean = m_mean;
		}
		
		iter = vp->begin();
		for(j=0; j<vp->size(); j++,iter++)
		{
			if((*iter)->pj_overlaped_objects.size()>0)
			{
				float x1 = (*iter)->m_mean.x;
				float y1 = (*iter)->m_mean.y;
				for(int l=0; l<(*iter)->pj_overlaped_objects.size(); l++)
				{
					Target * p_vp = (Target *)(*iter)->pj_overlaped_objects[l];
					float x2 = p_vp->m_mean.x;
					float y2 = p_vp->m_mean.y;
					cvLine(dst_image, cvPoint(x1,y1),cvPoint(x2,y2),CV_RGB(0,0,0),1);
				}
			}

			float w=(*iter)->get_width()*(*iter)->m_mean.s_w;
			float h=(*iter)->get_height()*(*iter)->m_mean.s_h;
			CvPoint pt_top_left,pt_bottom_right;
			pt_top_left.x = cvRound((*iter)->m_mean.x-0.5*w);
			pt_top_left.y = cvRound((*iter)->m_mean.y-0.5*h);
			pt_bottom_right.x = pt_top_left.x + w - 1;
			pt_bottom_right.y = pt_top_left.y + h - 1;
			cvRectangle(dst_image,pt_top_left, pt_bottom_right,CV_RGB(0,0,0),2);

			CvPoint pt_top_right,pt1,pt2;
			char text[255];int text_length=0;
			text_length = sprintf(text,"%d",(*iter)->get_id());
			int width = (mo_font_size.width+1)*text_length;
			int height = mo_font_size.height+1;
			pt_top_right.x = pt_bottom_right.x; 
			pt_top_right.y = pt_top_left.y;
			pt2.x= pt_top_right.x + 5 + width;// + 2;
			pt2.y= pt_top_right.y - 5;
			float xx=pt2.x,yy=pt2.y;
			pt2.x = xx; pt2.y = yy;
			pt1.x = pt2.x - width - 2; pt1.y = pt2.y;
			pt2.y = pt2.y + height + 2;
			cvLine(dst_image,pt_top_right,pt1,CV_RGB(10,10,10));
			cvRectangle( dst_image, pt1, pt2,CV_RGB(255,255,255),CV_FILLED);
			cvRectangle( dst_image, pt1, pt2,CV_RGB(10,10,10));
			pt1.x+=1;pt1.y=pt2.y-1;
			cvPutText( dst_image,(char *)&text,pt1,&mo_font,CV_RGB(10,10,10));
		}
	}

protected:
	unsigned char ** get_random_ptrs(){return m_random_ptrs2;}
	CvSample * get_sample_ptr(){return m_sample_ptr2;}
	int get_particle_bytes(void){return sizeof(CvSample);}
	CvSample * get_particle_ptr(){return get_sample_ptr();}
	CvSample * get_temp_particle_ptr(){return m_temp_particle_ptr2;}
	int get_particle_set_length(int start=0){return m_max_sample_size-start;}
	void set_subset_default_particle_size(int s){m_subset_default_particle_size=s;}
	int get_subset_default_particle_size(){return m_subset_default_particle_size;}

	void * get_variable_parameter_of_current_subset()
	{
		return m_target_of_current_subset;
	}
	void _build_subset_ptrs(int start, void * p_vp=0)
	{
		assert(start>=0 && start<m_max_sample_size);
		if(m_last_start!=start)
		{
			int i,N_R = m_sample_ptr[0].get_random_dimensions();
			for(i=0;i<N_R;i++)
			{
				m_random_ptrs2[i]=m_random_ptrs[i]+start*sizeof(float);
			}
			m_sample_ptr2 = m_sample_ptr+start;
			m_last_start=start;
		}
		m_target_of_current_subset = p_vp;
	}

	void reset_particle_ptrs(int length)
	{
		CvSample ** particle_ptrs = get_particle_ptrs();
		CvSample * particle_ptr = get_particle_ptr();
		unsigned char * tp = (unsigned char *)particle_ptr;
		const int particle_bytes = get_particle_bytes();
		for(int i=0; i<length; i++)
		{
			particle_ptrs[i]=(CvSample *)(tp + i*particle_bytes);
		}
	}
	int subset_set(int id, CvSample p)
	{
		std::vector<Target *>::iterator it;
		it = m_Targets.begin();
		for(int i=0; i<m_Targets.size(); i++,it++)
		{
			if((*it)->get_id()==id)
			{
				build_subset_ptrs((*it)->get_subset_start());
				CvSample * particle_ptr = get_particle_ptr();
				for(int i=0;i<(*it)->get_length();i++)
					particle_ptr[i]=p;
				return 1;
			}
		}
		return 0;
	}
	void copy_particles(CvSample * src, CvSample * dst, int length=1)
	{
		assert(src && dst && length>=1);
		const int particle_bytes = get_particle_bytes();
		const unsigned char * src2= (unsigned char *)src;
		unsigned char * dst2 = (unsigned char *)dst;
		for (int l=0; l<particle_bytes*length; l++)	dst2[l]=src2[l];
	}
	std::vector<Target *> * get_targets(){return &m_Targets;}
	CvSample ** get_particle_ptrs(){return m_particle_ptrs2;}
	
	void build_subset_ptrs(int start, void * p_vp=0)
	{
		if(m_last_start!=start)
		{
			m_particle_ptrs2 = m_particle_ptrs+start;
			m_temp_particle_ptr2 = m_temp_particle_ptr+start;
			_build_subset_ptrs(start,p_vp);
		}
		m_target_of_current_subset=p_vp;
	}
	void mo_draw_rectangle( void * img, CvPoint pt1, CvPoint pt2, CvScalar color)
	{
		CvMat stub, *mat = (CvMat*)img;
		unsigned char * pData;
		int step;
		CvSize size;
		int x_min,x_max,y_min,y_max;
		double buf[4];

		if (img==0) return;

		mat = cvGetMat( img, &stub, &step );
		cvScalarToRawData( &color, buf, mat->type, 0 );
		cvGetRawData( mat, (uchar**)&pData, &step, &size );

		if (!pData) return;

		x_min = MIN(pt1.x,pt2.x);
		x_max = MAX(pt1.x,pt2.x);
		y_min = MIN(pt1.y,pt2.y);
		y_max = MAX(pt1.y,pt2.y);

		if(x_min<0 || x_max<0 || x_min>size.width || x_max>size.width 
			|| y_min<0 || y_max<0 || y_min>size.height || y_max>size.height)
		return;
		
		uchar cb = ((uchar*)buf)[0], cg = ((uchar*)buf)[1], cr = ((uchar*)buf)[2];
		#define  ICV_PUT_POINT()  \
        {                         \
            if(cb) p2[0] = cb;    \
            if(cg) p2[1] = cg;    \
            if(cr) p2[2] = cr;    \
        }
		int x,y;
		for(y =0;y<size.height;y++,pData +=step )
		{
			unsigned char * p=pData;
			if (y==y_min || y==y_max)
			{
				unsigned char * p2;
				int w = x_max - x_min +1;
				p2 = p + 3*(x_min);
				for(x=0;x<w;x++)
				{
					ICV_PUT_POINT();
					p2+=3;
				}
			}
			if (y>y_min && y<y_max)
			{
				unsigned char * p2;
				p2 = p + 3*(x_min);
				ICV_PUT_POINT();
				p2 = p + 3*(x_max);
				ICV_PUT_POINT();
			}
		}
		#undef ICV_PUT_POINT
	}
	
	void sort_particle_ptrs_by_weight(int length)
	{
		_sort_particles_by_weight(get_particle_ptrs(),length,0);
	}
	CV_IMPLEMENT_QSORT( _sort_particles_by_weight, CvSample *, cmp_particle_by_weight)

public:
	CvMat* image;
	float m_system_variance[4];
	float m_screen_width;
	float m_screen_height;
	int hue_max;
	CvFont mo_font;
	CvSize mo_font_size;

protected:
	int m_max_sample_size;
	CvRandomGenerator m_rg;

	CvRandomGenerator m_rg2;
	int m_lut_size;
	int m_particle_size;
	int m_max_number_of_object;

	unsigned char ** m_random_ptrs;
	CvSample * m_sample_ptr;
	unsigned char * m_buffer;
	unsigned char ** m_random_ptrs2;
	CvSample * m_sample_ptr2;

	int m_last_start;
	int m_sample_set_buffer_size;
	void * m_target_of_current_subset;
	
	unsigned char * m_particle_filter_buffer;
	std::vector<Target*> m_Targets;
	CvSample * m_temp_particle_ptr;
	CvSample ** m_particle_ptrs;
	CvSample ** m_particle_ptrs2;
	CvSample * m_temp_particle_ptr2;
	int m_subset_default_particle_size;
};
